/* -*- mode: vala; c-basic-offset: 2; indent-tabs-mode: nil -*- */

[CCode (has_target = false)]
private delegate List<Unity.AbstractScope> LoadScopesFunction () throws Error;
[CCode (has_target = false)]
private delegate int GetVersionFunction ();

List<Unity.AbstractScope> load_module (string filename) throws Error
{
  Module module = Module.open (filename, ModuleFlags.BIND_LAZY);
  if (module == null)
  {
    throw new GLib.IOError.FAILED (@"Could not load module '$filename'");
  }

  void *function;
  if (!module.symbol ("unity_scope_module_get_version", out function))
  {
    throw new GLib.IOError.FAILED (@"Could not find 'get_version' symbol in '$filename'");
  }
  var get_version = (GetVersionFunction)function;
  if (get_version () != Unity.SCOPE_API_VERSION)
  {
    throw new GLib.IOError.FAILED (@"Plugin '$filename' is for wrong Scope API version");
  }

  if (!module.symbol ("unity_scope_module_load_scopes", out function))
  {
    throw new GLib.IOError.FAILED (@"Could not find 'load_scopes' symbol in '$filename'");
  }

  // Since we're executing code within the module at this point, it
  // should not be unloaded.
  module.make_resident ();
  var load_scopes = (LoadScopesFunction)function;
  return load_scopes ();
}

public int main (string[] args)
{
  // Build up a set of modules referenced by the scopes passed on the
  // command line, removing duplicates (as might happen when there are
  // multiple scopes implemented within a single plugin).
  var modules = new HashTable<string,int> (str_hash, str_equal);
  for (var i = 1; i < args.length; i++)
  {
    var scope_id = args[i];
    try
    {
      var metadata = Unity.Protocol.ScopeRegistry.ScopeMetadata.for_id (scope_id);
      if (metadata.module != null)
        modules[metadata.module] = 1;
    }
    catch (Error err)
    {
      error("%s", err.message);
    }
  }

  List<Unity.ScopeDBusConnector> connectors = null;
  modules.for_each ((filename, unused) =>
    {
      try
      {
        var module_scopes = load_module (filename);
        foreach (var scope in module_scopes)
        {
          var connector = new Unity.ScopeDBusConnector (scope);
          connector.export();
          connectors.append (connector);
        }
      }
      catch (Error err)
      {
        error("%s", err.message);
      }
    });
  Unity.ScopeDBusConnector.run ();
  return 0;
}
